@@ -1,27 +1,11 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "ClientInit.hxx"
 #include "Init.hxx"
+#include "Error.hxx"
 #include "Callback.hxx"
 #include "thread/Mutex.hxx"
-#include "util/RuntimeError.hxx"
 
 #include <upnptools.h>
 
@@ -52,8 +36,7 @@ DoInit()
 	auto code = UpnpRegisterClient(UpnpClientCallback, nullptr,
 				       &upnp_client_handle);
 	if (code != UPNP_E_SUCCESS)
-		throw FormatRuntimeError("UpnpRegisterClient() failed: %s",
-					 UpnpGetErrorMessage(code));
+		throw Upnp::MakeError(code, "UpnpRegisterClient() failed");
 }
 
 UpnpClient_Handle
@@ -62,7 +45,7 @@ UpnpClientGlobalInit(const char* iface)
 	UpnpGlobalInit(iface);
 
 	try {
-		const std::scoped_lock<Mutex> protect(upnp_client_init_mutex);
+		const std::scoped_lock protect{upnp_client_init_mutex};
 		if (upnp_client_ref == 0)
 			DoInit();
 	} catch (...) {
@@ -78,7 +61,7 @@ void
 UpnpClientGlobalFinish() noexcept
 {
 	{
-		const std::scoped_lock<Mutex> protect(upnp_client_init_mutex);
+		const std::scoped_lock protect{upnp_client_init_mutex};
 
 		assert(upnp_client_ref > 0);
 		if (--upnp_client_ref == 0)
