@@ -1,21 +1,5 @@
-/*
- * Copyright 2003-2021 The Music Player Daemon Project
- * http://www.musicpd.org
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+// SPDX-License-Identifier: GPL-2.0-or-later
+// Copyright The Music Player Daemon Project
 
 #include "Pack.hxx"
 #include "util/ByteOrder.hxx"
@@ -59,8 +43,7 @@ ConstructS24(uint8_t low, uint8_t mid, uint8_t high) noexcept
 /**
  * Read a packed signed little-endian 24 bit integer.
  */
-gcc_pure
-static int32_t
+static constexpr int32_t
 ReadS24LE(const uint8_t *src) noexcept
 {
 	return ConstructS24(src[0], src[1], src[2]);
@@ -69,8 +52,7 @@ ReadS24LE(const uint8_t *src) noexcept
 /**
  * Read a packed signed big-endian 24 bit integer.
  */
-gcc_pure
-static int32_t
+static constexpr int32_t
 ReadS24BE(const uint8_t *src) noexcept
 {
 	return ConstructS24(src[2], src[1], src[0]);
@@ -79,7 +61,7 @@ ReadS24BE(const uint8_t *src) noexcept
 /**
  * Read a packed signed native-endian 24 bit integer.
  */
-gcc_pure
+[[gnu::pure]]
 static int32_t
 ReadS24(const uint8_t *src) noexcept
 {
